@@ -188,7 +188,11 @@ class ArchiveFeed:
 
         update_download_metadata(remote_response, self.path)
 
-        return list(updated_items.keys()), download_paths
+        return list(updated_items.keys()), {
+            # Return values fit for CLI output
+            download_url: str(download_path)
+            for download_url, download_path in download_paths.items()
+        }
 
     def download_item_content(self, remote_format, remote_item_elem, remote_item_id):
         """
@@ -261,19 +265,40 @@ class ArchiveFeed:
 
         Also do any pre-processing needed to start updating the archive.
         """
-        if not self.archive.recreate and self.path.exists():
+        archive_tree = None
+        if (
+            not self.archive.recreate
+            and self.path.exists()
+            and self.path.read_text().strip()
+        ):
             logger.debug("Parsing archive XML: %r", self.url)
             with self.path.open() as feed_archive_opened:
-                archive_tree = etree.parse(feed_archive_opened)
-            archive_format = formats.FeedFormat.from_tree(self, remote_tree)
-            if not isinstance(archive_format, type(remote_format)):  # pragma: no cover
-                raise NotImplementedError(
-                    f"Remote feed format, {type(remote_format).__name__!r}, is "
-                    f"different from archive format, {type(archive_format).__name__!r}."
-                )
-            archive_root = archive_tree.getroot()
-            archived_items_parent = remote_format.get_items_parent(archive_root)
-        else:
+                # Try to parse the local archive version of the feed if possible.  If
+                # there are errors parsing it, then treat it as if it's the first time
+                # archiving this feed.
+                try:
+                    archive_tree = etree.parse(feed_archive_opened)
+                except SyntaxError:
+                    logger.exception(
+                        "Unhandled exception parsing archive feed: %r",
+                        self.url,
+                    )
+                    if utils.POST_MORTEM:  # pragma: no cover
+                        raise
+            if archive_tree is not None:
+                archive_format = formats.FeedFormat.from_tree(self, remote_tree)
+                if not isinstance(
+                    archive_format,
+                    type(remote_format),
+                ):  # pragma: no cover
+                    raise NotImplementedError(
+                        f"Remote feed format, {type(remote_format).__name__!r}, is "
+                        "different from archive format, "
+                        f"{type(archive_format).__name__!r}."
+                    )
+                archive_root = archive_tree.getroot()
+                archived_items_parent = remote_format.get_items_parent(archive_root)
+        if archive_tree is None:
             # First time requesting this feed, copy the remote feed, minus the items to
             # the archive and download the feed-level assets.
             logger.info(
@@ -693,392 +718,6 @@ class ArchiveFeed:
         ] = str(content_link_path)
         return content_link_path
 
-    def migrate(
-        self, target_path
-    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
-        """
-        Use archived feed XML to migrate items as they would be handled now.
-        """
-        # Request and parse the remote feed XML for the most current representation of
-        # items that are still available in the remote.
-        remote_response = self.archive.requests.get(self.url)
-        remote_tree = self.load_remote_tree(remote_response)
-        remote_root = remote_tree.getroot()
-        remote_format = formats.FeedFormat.from_tree(self, remote_tree)
-
-        # Find the old feed XML file in the original archive, starting with the oldest
-        # possible version first
-        for redirect_response in [remote_response] + list(
-            reversed(remote_response.history),
-        ):
-            remote_response_relative = self.archive.response_to_path(
-                remote_response,
-                request=redirect_response.request,
-            )
-            # Try without extension as before working around MIME types issues
-            orig_feed_relative = remote_response_relative.with_suffix("")
-            self.path = self.archive.root_path / orig_feed_relative
-            if self.path.is_file():  # pragma: no cover
-                break
-            # Fallback to the currenct/correct extension
-            orig_feed_relative = remote_response_relative
-            self.path = self.archive.root_path / orig_feed_relative
-            if self.path.is_file():  # pragma: no cover
-                break
-        else:  # pragma: no cover
-            raise ValueError(
-                "Could not find original archive feed file from URL: "
-                f"{self.url!r} -> {str(orig_feed_relative)!r}"
-            )
-
-        # Copy the feed XML to the location the current version of the code would
-        # generate
-        migrated_feed_relative = self.archive.response_to_path(remote_response)
-        migrated_paths = {}
-        migrated_feed_path = self.archive.migrate_path(
-            target_path,
-            orig_feed_relative,
-            migrated_feed_relative,
-            copy=True,
-        )
-        migrated_paths[self.url] = (str(self.path), str(migrated_feed_path))
-
-        # Parse the feed XML in the target archive
-        logger.debug("Parsing archive XML: %r", self.url)
-        with migrated_feed_path.open() as migrated_feed_opened:
-            archive_tree = etree.parse(migrated_feed_opened)
-        archive_format = formats.FeedFormat.from_tree(self, archive_tree)
-        archive_root = archive_tree.getroot()
-        self_link_elems = archive_format.get_items_parent(archive_root).xpath(
-            archive_format.SELF_LINK_XPATH,
-        )
-
-        remote_item_elems_list = list(remote_format.iter_items(remote_root))
-        archive_item_elems = list(archive_format.iter_items(archive_root))
-        if (
-            len(remote_item_elems_list) - len(archive_item_elems)
-        ) > 4:  # pragma: no cover
-            logger.warning(
-                "Many more items in remote than archive: %s > %s",
-                len(remote_item_elems_list),
-                len(archive_item_elems),
-            )
-
-        # Link feed-level downloads into the target archive
-        migrated_paths.update(
-            self.migrate_url_results(
-                target_path,
-                xpath_queries=archive_format.DOWNLOAD_FEED_URLS_XPATHS,
-                archive_elem=archive_root,
-                remote_elem=remote_root,
-                exclude_url_elems=self_link_elems,
-            ),
-        )
-
-        # Map remote items to their IDs for pairing with archive items
-        remote_item_elems = {}
-        for remote_item_elem in remote_item_elems_list:
-            remote_item_elems[
-                remote_format.get_item_id(remote_item_elem)
-            ] = remote_item_elem
-
-        # Link item-level downloads into the target archive
-        archived_items_parent = archive_format.get_items_parent(archive_root)
-        for archive_item_elem in archive_item_elems:
-            archive_item_id = archive_format.get_item_id(archive_item_elem)
-            remote_item_elem = remote_item_elems.get(archive_item_id)
-
-            # Migrate item assets in the archive only
-            migrated_paths.update(
-                self.migrate_url_results(
-                    target_path,
-                    xpath_queries=archive_format.DOWNLOAD_ITEM_ASSET_URLS_XPATHS,
-                    archive_elem=archive_item_elem,
-                    remote_elem=remote_item_elem,
-                ).items()
-            )
-
-            # Migrate item content in the archive and re-link
-            migrated_content_paths = self.migrate_url_results(
-                target_path,
-                xpath_queries=archive_format.DOWNLOAD_ITEM_CONTENT_URLS_XPATHS,
-                archive_elem=archive_item_elem,
-                remote_elem=remote_item_elem,
-            )
-            migrated_paths.update(migrated_content_paths)
-            # Remove content links prior to re-creating
-            for item_content_url_result in formats.all_xpaths_results(
-                archive_item_elem,
-                archive_format.DOWNLOAD_ITEM_CONTENT_URLS_XPATHS,
-            ):
-                # Remove content links listed in the download's item XML namespace
-                # attributes.
-                content_elem = item_content_url_result.getparent()
-                content_link_strs = []
-                for attr_name, content_link_str in content_elem.attrib.items():
-                    if not attr_name.startswith(f"{{{self.NAMESPACE}}}content-link-"):
-                        continue
-                    content_link_strs.append(content_link_str)
-                    del content_elem.attrib[attr_name]
-                # Remove content links that may be stale/old from previous versions of
-                # the code
-                content_archive_relative = pathlib.PurePosixPath(
-                    urllib.parse.unquote(
-                        urllib.parse.urlsplit(item_content_url_result).path.lstrip("/"),
-                    ),
-                )
-                for link_path_plugin in self.link_path_plugins:
-                    for content_link_path in self.list_item_content_link_plugin_paths(
-                        archive_format.get_items_parent(archive_root),
-                        archive_item_elem,
-                        item_content_url_result,
-                        content_archive_relative.name,
-                        content_archive_relative,
-                        link_path_plugin,
-                    ):
-                        content_link_str = str(content_link_path)
-                        content_link_strs.extend(
-                            [
-                                # Fully unquoted
-                                content_link_str,
-                                # Double quoted path characters, single quoted others
-                                urllib.parse.quote(
-                                    utils.quote_path(content_link_str),
-                                    safe="/ ",
-                                ),
-                                # Single quoted all characters
-                                urllib.parse.quote(content_link_str, safe="/ "),
-                                # Double quoted all characters
-                                urllib.parse.quote(
-                                    urllib.parse.quote(content_link_str, safe="/ "),
-                                    safe="/ ",
-                                ),
-                                # Double quoted path characters
-                                utils.quote_path(utils.quote_path(content_link_str)),
-                            ]
-                        )
-                # Perform the actual deletion
-                logger.debug(
-                    "Cleaning up content link viriations:\n%s",
-                    "\n".join(content_link_strs),
-                )
-                for content_link_str in content_link_strs:
-                    content_link_path = pathlib.Path(content_link_str)
-                    content_link_stem = content_link_path.stem
-                    content_index = 0
-                    while content_link_path.is_symlink():
-                        logger.info(
-                            "Removing content link: %r -> %r",
-                            content_link_str,
-                            str(content_link_path.readlink()),
-                        )
-                        content_link_path.unlink()
-                        content_index += 1
-                        # Missing the dash before the index
-                        content_link_path = content_link_path.with_stem(
-                            f"{content_link_stem}{str(content_index)}",
-                        )
-                        if content_link_path.is_symlink():
-                            logger.info(
-                                "Removing content link: %r -> %r",
-                                content_link_str,
-                                str(content_link_path.readlink()),
-                            )
-                            content_link_path.unlink()
-                        content_link_path = content_link_path.with_stem(
-                            f"{content_link_stem}-{str(content_index)}",
-                        )
-            # Re-apply current `link-path` plugins
-            self.link_item_content(
-                feed_elem=archived_items_parent,
-                item_elem=archive_item_elem,
-                item_content_paths={
-                    archive_url_result: pathlib.Path(
-                        target_file_path,
-                    ).relative_to(target_path)
-                    for archive_url_result, (
-                        archive_file_path,
-                        target_file_path,
-                    ) in migrated_content_paths.items()
-                },
-            )
-
-        # Pretty format the feed for readability
-        etree.indent(archive_tree)
-        # Update the archived feed file
-        archive_tree.write(str(migrated_feed_path))
-
-        return len(migrated_paths)
-
-    def migrate_url_results(
-        self,
-        target_path,
-        xpath_queries,
-        archive_elem,
-        remote_elem=None,
-        exclude_url_elems=None,
-    ):  # pylint: disable=too-many-arguments,too-many-locals
-        """
-        Reconcile the same XPath query URLs from both the remote and the archive.
-        """
-        archive_url_results = formats.all_xpaths_results(archive_elem, xpath_queries)
-        remote_url_results = None
-        if remote_elem is not None:  # pragma: no cover
-            remote_url_results = formats.all_xpaths_results(remote_elem, xpath_queries)
-
-        # Compare the 2 sets of XPath URL results
-        if remote_url_results and len(remote_url_results) != len(
-            archive_url_results
-        ):  # pragma: no cover
-            logger.warning(
-                "Remote has different XPath results than the archive: %s != %s",
-                len(archive_url_results),
-                len(remote_url_results),
-            )
-            remote_url_results = None
-        if remote_url_results:
-            for archive_url_result_idx, archive_url_result in enumerate(
-                archive_url_results,
-            ):
-                archive_url_elem = archive_url_result.getparent()
-                remote_url_result = remote_url_results[archive_url_result_idx]
-                remote_url_elem = remote_url_result.getparent()
-                if archive_url_elem.tag != remote_url_elem.tag:  # pragma: no cover
-                    logger.warning(
-                        "Remote has different XPath results than the archive: %r",
-                        archive_url_result,
-                    )
-                    remote_url_results = None
-                    break
-
-        # Perform the actual migrations
-        if exclude_url_elems is None:
-            exclude_url_elems = []
-        file_paths = {}
-        for archive_url_result_idx, archive_url_result in enumerate(
-            archive_url_results,
-        ):
-            archive_url_elem = archive_url_result.getparent()
-            for exclude_url_elem in exclude_url_elems:
-                # Check if the parent element of the URL result is excluded
-                if exclude_url_elem is archive_url_elem:
-                    break
-            else:
-                remote_url_result = None
-                if remote_url_results:  # pragma: no cover
-                    remote_url_result = remote_url_results[archive_url_result_idx]
-                migrated_file_paths = self.migrate_url_result(
-                    target_path,
-                    archive_url_result,
-                    remote_url_result,
-                )
-                if migrated_file_paths is None:  # pragma: no cover
-                    break
-                archive_file_path, target_file_path = migrated_file_paths
-                file_paths[archive_url_result] = (
-                    str(archive_file_path),
-                    str(target_file_path),
-                )
-        return file_paths
-
-    def migrate_url_result(
-        self,
-        target_path,
-        archive_url_result,
-        remote_url_result=None,
-    ):
-        """
-        Use `feed-archiver` XML attributes to migrate an individual URL.
-        """
-        # Lookup the original URL from the XML namespace attribute
-        url_parent = archive_url_result.getparent()
-        if remote_url_result is not None:
-            remote_url = remote_url_result
-        elif archive_url_result.attrname:  # pragma: no cover
-            remote_url = url_parent.attrib[
-                f"{{{self.NAMESPACE}}}attribute-{archive_url_result.attrname}"
-            ]
-        else:  # pragma: no cover
-            remote_url = url_parent.attrib[f"{{{self.NAMESPACE}}}text"]
-        remote_url_split = self.resolve_url(remote_url)
-        target_relative_path = self.archive.url_to_path(
-            remote_url_split.geturl(),
-        )
-
-        # Determine the path to the file in the archive
-        archive_base_url_split = urllib.parse.urlsplit(
-            self.config.get("base-url") or self.archive.global_config["base-url"],
-        )
-        archive_url_split = urllib.parse.urlsplit(archive_url_result)
-        archive_relative_path = pathlib.PurePosixPath(
-            urllib.parse.unquote(archive_url_split.path.lstrip("/")),
-        )
-        # Start with the oldest form, relative to the feed
-        if (
-            not archive_url_split.scheme and not archive_url_split.netloc
-        ):  # pragma: no cover
-            archive_file_path = self.path.parent / archive_relative_path
-            archive_relative_path = archive_file_path.relative_to(
-                self.archive.root_path,
-            )
-        # Newer, absolute URLs using `base-url` from the config
-        elif (archive_url_split.scheme, archive_url_split.netloc) == (
-            archive_base_url_split.scheme,
-            archive_base_url_split.netloc,
-        ):
-            archive_relative_path = pathlib.PurePosixPath(
-                urllib.parse.unquote(archive_url_split.path.lstrip("/")),
-            )
-            archive_file_path = self.archive.root_path / archive_relative_path
-        else:  # pragma: no cover
-            # Probably a download URL added to the remote feed since the original
-            # archive was last updated
-            logger.debug(
-                "URL not downloaded in original archive: %r",
-                archive_url_split.geturl(),
-            )
-            return None
-        # Handle directory `index.html` files
-        if archive_file_path.is_dir():
-            archive_relative_path = archive_relative_path / self.archive.INDEX_BASENAME
-            archive_file_path = archive_file_path / self.archive.INDEX_BASENAME
-        else:
-            target_relative_path = target_relative_path.with_suffix(
-                archive_relative_path.suffix,
-            )
-
-        # Update the archived URL in the XML a
-        target_relative_url_path = target_relative_path
-        if target_relative_path.name == self.archive.INDEX_BASENAME:
-            target_relative_url_path = target_relative_path.parent
-        migrated_url_split = self.archive.url_split._replace(
-            path=str(
-                pathlib.PurePosixPath(self.archive.url_split.path)
-                / urllib.parse.quote(str(target_relative_url_path)),
-            ),
-        )
-        logger.info(
-            "Updating download path: %r -> %r",
-            str(archive_relative_path),
-            str(target_relative_path),
-        )
-        if archive_url_result.attrname:
-            url_parent.attrib[archive_url_result.attrname] = migrated_url_split.geturl()
-        else:
-            url_parent.text = migrated_url_split.geturl()
-
-        # Link into the migrated archive unless it's already been linked by a redundant
-        # item enclosure/content element.
-        target_file_path = target_path / target_relative_path
-        if not target_file_path.exists():
-            target_file_path = self.archive.migrate_path(
-                target_path,
-                archive_relative_path,
-                target_relative_path,
-            )
-
-        return archive_file_path, target_file_path
-
 
 def update_download_metadata(download_response, download_path):
     """
